// Copyright 2016 concept3D, Inc.

var modelName = "";
var venueName = "";
var roomName = "";
var mapId = 0;
var refId = "";
var atlasId = "";

$(document).ready(function() {
  create_bridge();
  activateConfigChange();
//  load_youtube_iframe_api();
  callback("html_ready");
});




//********************
//Helper functions
//********************

//====================================================================
// load client-specific styles
//--------------------------------------------------------------------
function loadCSS(clientDir) {
  debug("css="+clientDir);
  $('<link/>', {
    id: 'client-css',
    href: clientDir,
    rel: 'stylesheet'
    }).appendTo(document.head);
}

// Utility method to call back to Ruby. The payload (an optional JSON object) is
// stored in #SU_BRIDGE so Ruby can use UI::WebDialog.get_element_value to get it.
function callback(name, data) {
  // Defer with a timer in order to allow the UI to update.
  setTimeout(function() {
    var $bridge = $("#SU_BRIDGE");
    $bridge.text("");
    if (data !== undefined) {
      var json = JSON.stringify(data);
      $bridge.text(json);
    }
    window.location = "skp:callback@" + name;
  }, 0);
}


// Simple function to capitalize the first letter of a string. Thanks: http://stackoverflow.com/questions/1026069/capitalize-the-first-letter-of-string-in-javascript
String.prototype.capitalizeFirstLetter = function() {
    return this.charAt(0).toUpperCase() + this.slice(1);
}

// Function to serialize the elements of an object into text
$.fn.serializeObject = function()
{
    var o = {};
    var a = this.serializeArray();
    $.each(a, function() {
        if (o[this.name] !== undefined) {
            if (!o[this.name].push) {
                o[this.name] = [o[this.name]];
            }
            o[this.name].push(this.value || '');
        } else {
            o[this.name] = this.value || '';
        }
    });
    return o;
};



//********************
//MapSpacer functions
//********************

//** MapSpacer calls from the UI **//
function changeRoomType(e){
  try {
    addPreloader(e);
    toggleLayer(e.currentTarget.name, true);
  } catch (e) {
    log_error(e);
  }
}

function addPreloader(e){
  removeAllPreloaders();
  $(e.currentTarget).append("<img class='preloader'></img>");
}

function removeAllPreloaders(){
  $(".preloader").remove();
}

function toggleLayer(layerKeyword, finalState){
  try {
    var json = { };
    json[layerKeyword] = finalState;
    callback("toggle_layer", json);
  } catch (e) {
    log_error(e);
  }
}

function layerOff(e){
  try {
    addPreloader(e);

    // Toggle layer in SketchUp to same finalState
    toggleLayer(e.currentTarget.name, false);
  } catch (e) {
    log_error(e);
  }
}

function layerOn(e){
  try {
    addPreloader(e);

    // Toggle layer in SketchUp to same finalState
    toggleLayer(e.currentTarget.name, true);
  } catch (e) {
    log_error(e);
  }
}

function purger(){
  callback("purge_unused");
}

function toggleButton(e){
  try {
    addPreloader(e);
    // Toggle button and any children
    var finalState = toggleButtonHelper(e.currentTarget);

    // Toggle layer in SketchUp to same finalState
    toggleLayer(e.currentTarget.name, finalState);
  } catch (e) {
    log_error(e);
  }
}

function toggleLayerButtonUnique(e){
  try {
    addPreloader(e);
    // Toggle off all buttons of the parent section
    var finalState = toggleButtonUniqueHelper(e.currentTarget)

    // Toggle layer in SketchUp to same finalState
    toggleLayer(e.currentTarget.name, finalState);
  } catch (e) {
    log_error(e);
  }

}

function toggleButtonUniqueHelper(target){
    // Toggle off all buttons of the parent section
  $(target).parent('div').children('button').toggleClass('active', false);

  // Then turn on the event button and any children
  var finalState = toggleButtonHelper(target, true);

  return finalState;
}

function toggleButtonHelper(target, finalState){
  // If finalState was not passed, determine current state
  if (finalState === undefined) {
    finalState = !$(target).hasClass('active');
  }
  // Toggle target and any children
  $(target).toggleClass('active', finalState);
  $(target).children('div.button-switch').toggleClass('active', finalState);

  // If the Select All button was clicked, then have all the subrooms update active status
  if(target.name == "subroom_all") {
    $('.breakout-inner button').toggleClass('active', finalState);
  }
  return finalState;
}

function activateTool(e){
  try {
    // Toggle off all buttons of the parent section
    var finalState = toggleButtonUniqueHelper(e)

    toolName = e.currentTarget.name;
    var json = { };
    json['tool'] = toolName;
    callback("activate_tool", json);
  } catch (e) {
    log_error(e);
  }
}

function showDCOptions(e){
  try {
    var finalState = toggleButtonHelper(e.currentTarget)
    var json = { };
    json['state'] = finalState;
    callback("show_dc_options", json);
  } catch (e) {
    log_error(e);
  }
}

function takeScreenshot(){
  try {
    callback("take_screenshot");
  } catch (e) {
    log_error(e);
  }
}

function openFolder(){
  try {
    callback("open_folder");
  } catch (e) {
    log_error(e);
  }
}

function exportReport(){
  try {
    callback("export_report");
  } catch (e) {
    log_error(e);
  }
}

function rotateSelection(e){
  var selectionName = e.currentTarget.name;
  var json = { };
  json['rotate'] = selectionName;
  callback("rotate_selection", json);
}

function stopProp(e){
  if(e.preventDefault) {
      e.preventDefault();
  }

  if(e.stopPropagation) {
      e.stopPropagation();
  }
}

function importSkp(e) {
  var selectionName = e.currentTarget.name;
  var json = { };
  json['target'] = selectionName;
  callback("import_skp", json);
}

function loadModel(e){
  addPreloader(e);
  var name = e.currentTarget.name;
  var json = { };
  json["model"] = name;
  callback("load_model", json);
}

function changeView(e){
  var viewName = e.currentTarget.name;
  var json = { };
  json["view"] = viewName;
  callback("change_view", json);
}

function openRfp(){
  var json = { };
  json["atlasId"] = atlasId;
  callback("open_RFP", json);
}

function openYouTube(){
  var json = { };
  json["atlasId"] = atlasId;
  callback("open_YouTube", json);
}

function openSite(){
  callback("open_site");
}

function openEW(string){
  // Redirect to special skp action within SU and it'll open new window to EW
  document.location.href = "skp:launchEW@"+string;
}

function openMapSpacer(){
  callback("open_mapspacer");
}

function openExternalWebpage(e){
  var url = e.currentTarget.attributes["alt"].value;
  var json = { };
  json["url"] = url;
  callback("open_external_webpage", json);
}

function activateConfigChange(){
  $('#config-form').change(function(){
    try {
      var json = $("#config-form").serializeObject();
      callback("change_config", json);
    } catch (e) {
      log_error(e);
    }
  });
}

function changeMediaFolder(){
  callback("change_media_folder");
  $( ".media.accordion .ui-accordion-header span" ).click();
}

function updateStatistics(){
  callback("update_statistics");
}
//** MapSpacer calls from Ruby **//

function updateOutput(jsonObject) {
  try {
    var string = "";

    $.each($.parseJSON(jsonObject), function (category, value) {
      if(value) {
        $.each(value, function (i, object) {
          string += "<div class=row>";
            string +=  "<div class=value>" + object.count + "</div>";
            string +=  "<div class=label>" + object.description;
            if (object.attributes != "" ) string +=  " ("+object.attributes+")";
            //string +=  " ("+object.name+")</div>";
            string +=  "</div>";
          string += "</div>";
        });
      }
    });
    $("#output").html(string);
  } catch (e) {
    log_error(e);
  }

  // Assuming this is the last call back from Ruby after performing an action
  // then we can remove all preloader images
  removeAllPreloaders();
}

function updateParametrics(string) {
  $(".breakout-inner").prepend(string);
}

function updateConfigurations(string) {
  $(".room-set-inner").html(string);
}

function updateModelStatus(json) {
  try {
    modelName = json['modelName'];
    venueName = json['venueName'];
    roomName = json['roomName'];
    mapId = json['mapId'];
    refId = json['refId'];
    $("#header h1").html(modelName);
    $("#menu-window .main-sub button[name~='rfp']").html(modelName);
    $('#mapspacer-version').html(json['MapSpacer Version']);
    $('#venue-versions-contents').loadTemplate($('#venue-versions-template'), json['Venue Extensions']);

    toggleButtonHelper("#sidebar .toggle button[name~='3D']", json['3D']);
    toggleButtonHelper("#sidebar .toggle button[name~='3D_Walls']", json['3D Walls']);
    toggleButtonHelper("#sidebar .toggle button[name~='Utilities']", json['Utilities']);

    $.each(json['subrooms'], function (index, status) {
      toggleButtonHelper("#content .breakout button[name~='subroom_"+status['name']+"']", status['visible']);
    });

    // if there are no subrooms to choose from, then hide the entire section
    if (json['subrooms'].length < 2) {
      $(".breakout").hide();
    } else {
      $(".breakout").show();
    }

    toggleButtonUniqueHelper("#sidebar .tools button[name~='"+json['currentTool']+"']");

    $('.breakout-error').toggle(!json['validConfiguration']);

    // Fill in config form
    var config = JSON.parse(json['config']);
    var option = (config.recalculate)?1:0;
    $('#configuration input[name=recalculate]')[option].checked = true;
    var option = (config.airwalls)?1:0;
    $('#configuration input[name=airwalls]')[option].checked = true;

    // Now that we have a modelName, get resource info
    get_atlas_info();

    hideVenueInstallButton();
  } catch (e) {
    log_error(e);
  }
}

function updateAnnouncements(string) {
  //$(".announcementsText").html(string);
  //$('#announcements').show();
}

function updateTrainingTab(string) {
  //$("#tabs-instructions .training ul").html(string);
}

function updateMedia(string) {
  $("#content .media .icons").html(string);
  updatePagination();
}


function componentSelected(selected) {
  $("#sidebar .rotate button").attr("disabled", !selected);
}


function updateComponents(string) {
  $("#content .components .icons").html(string);
  updatePagination();
}


//** Helper functions **//

function debug(string) {
  callback("debug", { 'message' : string });
}

//noinspection JSUnusedGlobalSymbols
function log_error(error) {
  if (error.name == 'SyntaxError') {
    debugger; // TODO(thomthom): Don't call this in release.
  }
  //noinspection JSUnresolvedVariable
  var data = {
    'code'      : error.number,
    'filename'  : error.fileName,     // Missing in IE.
    'name'      : error.name,
    'line'      : error.lineNumber,   // Missing in IE.
    'col'       : error.columnNumber, // Missing in IE.
    'stack'     : error.stack,
    'string'    : error.toString()
  };
  callback("log_error", data);
}


// Creates a hidden textarea element used to pass data from JavaScript to
// Ruby. Ruby calls UI::WebDialog.get_element_value to fetch the content and
// parse it as JSON.
// This avoids many issues in regard to transferring data:
// * Data can be any size.
// * Avoid string encoding issues.
// * Avoid evaluation bug in SketchUp when the string has a single quote.
function create_bridge() {
  var $bridge = $("<textarea id='SU_BRIDGE'></textarea>");
  $bridge.hide();
  $("body").append($bridge);
}


function is_a_string(object) {
  return object instanceof String || typeof object == 'string';
}


function loading_image() {
  return '../images/ajax-loader.gif';
}

function missing_image() {
  return '../images/missing-image.png';
}

//*****************************
// Google Api Functions
//******************************

// function googleApiClientReady() {
//   console.log('Google Api Loaded');
//   gapi.client.setApiKey('AIzaSyC_tgQ9nC3hGGgxoKqm8-D1_TuXP4dgM50');

  // PLFA727FF8F2B1C4A2 is the playlist ID for Concept3D's Sixty Second Sketchup Series youtube playlist.
  // var playlistId = 'PLFA727FF8F2B1C4A2';

  // // gapi.client.load('youtube', 'v3', function () {
  //   // NOTE: This callback never seems to get called, for reasons that I have yet to find out.
  //   console.log('Youtube Api Loaded');
  //   requestYoutubePlaylist(playlistId);
  // });

  // request.then(function (resp) {
  //   console.log('Youtube Api Loaded');
  //   requestYoutubePlaylist(playlistId);
  // }, function (reason) {
  //   console.log('Youtube Api Loaded');
  //   youtubeApiLoadError(reason);
  // });

// }

// function requestYoutubePlaylist (playlistId) {
//   $('#tutorial-player').html('');

  // var requestOptions = {
  //   playlistId: playlistId,
  //   part: 'snippet',
  //   maxResults: 50,
  // };

  // var request = gapi.client.youtube.playlistItems.list(requestOptions);
  // request.then(function(resp) {
  //   handlePlaylistResponse(resp);
  // }, function (reason) {
  //   $('#tutorial-player').html('There was an error retrieving the tutorial playlist from Youtube: ' + reason);
  // });
// }

// function youtubeApiLoadError (reason) {
//   $('#tutorial-player').html('There was an error loading the youtube API: ' + reason);
// }

// function handlePlaylistResponse (response) {
//   nextPageToken = response.result.nextPageToken;
//   var playlistItems = response.result.items;
//   if (playlistItems) {
//     $.each(playlistItems, function(index, item) {
//       displayResult(item.snippet);
//     });
//   } else {
//     $('#tutorial-player').html('There was an error retrieving the tutorial playlist from Youtube.');
//   }
// }

// function youtubeRequestPlaylistError (reason) {
//   $('#tutorial-player').html('There was an error requesting the tutorial playlist from Youtube: ' + reason);
// }

// function displayResult(videoSnippet) {
//   var title = videoSnippet.title;
//   var videoId = videoSnippet.resourceId.videoId;
//   $('#tutorial-player').append('<p>' + title + ' - ' + videoId + '</p>');
// }


//*****************************
// Youtube Iframe Api Functions
//******************************

function load_youtube_iframe_api() {
  var tag = document.createElement('script');

  tag.src = "https://www.youtube.com/iframe_api";
  var firstScriptTag = document.getElementsByTagName('script')[0];
  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);
}

function onYouTubeIframeAPIReady() {
  var tutorialPlayer;

    function YoutubeVideoClass(title, id) {
      this.title = title;
      this.id = id;
    };
    YoutubeVideoClass.constructor = YoutubeVideoClass;

    YoutubeVideoClass.prototype.makeMany = function (videos) {
      var youtubeVideos = [];
      for (var i in videos) {
        var video = videos[i];
        var newVideo = new YoutubeVideoClass(video[0], video[1]);
        youtubeVideos.push(newVideo);
      }
      return youtubeVideos;
    };

    YoutubeVideoClass.prototype.renderHTML = function () {
      var self = this;
      var selector = self.getSelector();
      var templateOptions = { append: true, afterInsert: function () {
        $(selector).parent('.tutorial-body').hide();
        self.loadPlayer();
        self.registerClickHandler();
      }};
      $('#tutorial-videos').loadTemplate($('#tutorial-template'), { title: self.title, id: self.id }, templateOptions);
    };

    YoutubeVideoClass.prototype.loadPlayer = function () {
        this.tutorialPlayer = new YT.Player(this.id, {
        height: '390',
        width:'640',
        videoId: this.id,
        events: {
          'onReady': onPlayerReady,
        }
      });
    };

    YoutubeVideoClass.prototype.registerClickHandler = function () {
      var self = this;
      var selector = self.getSelector();
      $(selector).closest('.tutorial-header').click(function () {
        $(selector).closest('.tutorial-body').toggle();
      });
    };

    YoutubeVideoClass.prototype.getSelector = function () {
      return '#' + this.id;
    };
  var YoutubeVideo = new YoutubeVideoClass();

  var tutorialVideos = YoutubeVideo.makeMany([
      ['Getting Started','Tcs9V9GW1E8'],
      ['Faces, Edge, and Surfaces','D3amXjPCtiI'],
      ['Click-Release-Click vs. Click-Drag-Release','YVpwfPlkbsE'],
      ['Intuitive Mouse Navigation','CcQgEErF1xE'],
      ['SketchUp Free vs. SketchUp Pro','VnYV4881tHc'],
      ['Choosing and Creating a Template','fH8spf1QMzo'],
      ['Locking Inferences and References','tY9cPLg0GzE'],
      ['Automatic and Encouraged Inferencing','B_OfdvRuofs'],
      ['Geometry Stickyness, Groups, and Components','jNKCXdHKXT0'],
      ['Pre-Select vs. Active Select','PFdQhTHyAr8'],
      ['Perspective vs. Parallel Projection','6_YDXihyy14'],
      ['Making Copies with the Move tool','Sk_g2JqNrBk'],
      ['Organizing with Nested Components and The Outliner','7OBljxmDLSI'],
      ['A Good Reason to Color Segments \"By Axes\"','-XUZHZAnrjk'],
      ['Image, Texture, Matched Photo Import Eptions','gOB7fBrEjwY'],
      ['Using the Matched Photo Feature','9QtNI9C_68U'],
      ['Editing Textures with an External Editor','5xK5S1C951o'],
      ['Scaling, Distorting, Stretching and Otherwise Positioning Textures','vO16eaFpBF0'],
      ['Projected Textures vs. Not-Projected Textures','KFZ6Ts9SaT8'],
      ['Animating Section Cuts','Vtnryqm9jjo'],
      ['Boolean Operations with \"Intersect with Model\"','k-TOWz0Jczk'],
      ['Precise Sandbox Stamp Tool Stamping','XN44ZB98E5g'],
      ['Dynamic Components: A New Way to See, a New Way to Create','_EZV-BhRVbQ'],
      ['Dynamic Components, What Are They Good For?','lFIrb73KN8w'],
      ['Dimensions & Layout','cZfZeF1tsX8']
  ]);

  for (var i in tutorialVideos) {
    var video = tutorialVideos[i];
    video.renderHTML();
  }
}

function onPlayerReady(event) {
}

//=============================================================================
// Help's Q&A Templating
//-----------------------------------------------------------------------------

function load_qa() {
  $.ajax({
    url: "https://msapi.concept3d.com/sketchup/qa",
    type: 'GET',
    success: function(qa){
/*
      qa = { data: [
          {question : "Sample FAQ question about CampusBird goes here", answer : "And here's a sample answer to that FAQ question. It pops down just like that."},
          {question : "This is a sample CampusBird FAQ question", answer : "And here's a sample answer to that FAQ question. It pops down just like that."}
        ]
      };
*/
      //debug("successfully uploaded Q&A");
      var templateOptions ={}; // {append: true};
      //$('#qa-section').loadTemplate($('#qa-template'), qa.data, templateOptions);
    },
    error: function(xhr, status, err){
      //debug("failed to get Q&A: "+err.toString());
    }
  });

}

//=============================================================================
// Find Atlas location information
//-----------------------------------------------------------------------------

function get_atlas_info() {
  if (!atlasId && mapId && refId) {
    $.ajax({
      url: "https://api.concept3d.com/locations",
      type: 'GET',
      data: { ref: refId, map: mapId, key: '0001085cc708b9cef47080f064612ca5' },
      success: function(data){
        // Using jquery template: https://github.com/codepb/jquery-template
        atlasId = data.id;
        debug("atlasId fetched="+atlasId);
      },
      error: function(xhr, status, err){
        debug("failed to get Atlas ID: "+err.toString());
      }
    });

    $.ajax({
      url: "https://msapi.concept3d.com/spaces",
      type: 'GET',
      data: { code: refId, map: mapId },
      success: function(data){
        // Using jquery template: https://github.com/codepb/jquery-template
        $('#model-plan-contents').loadTemplate($('#resource-template'), data.documents.plan);
        $('#model-explore-contents').loadTemplate($('#resource-template'), data.documents.explore);
        $('#model-common-contents').loadTemplate($('#resource-template'), data.documents.common);
      },
      error: function(xhr, status, err){
        debug("failed to get MapSpacer information for " + roomName + ": " + err.toString());
      }
    });
  }
}

function showVenueInstallButton(string) {
  toggleMenu();
  $('#venue-update-block').show();
  $('#venue-update-name').html(string);
  $('#venue-update-block button').attr("onclick","openEW('mapspacer-"+string.toLowerCase()+"')");
  toggleMenuItem( {currentTarget: {name: 'updates'} } );
}

function hideVenueInstallButton() {
  $('#venue-update-block').hide();
}

$.addTemplateFormatter({
    IconFormatter : function(value, template) {
        if(value == "skp") {
          return "../images/icons/sketchup.svg";
        } else {
          return "../images/icons/pdf.svg";
        }
      },
    ResourceLocation : function(value, type) {
        //debug("value: "+value);
        if(value.includes("http")) {
          return value;
        } else {
          return "https://3dwarehouse.sketchup.com/model.html?id="+value;
        }
      }
});
